<?php
/**
 * Created by PhpStorm.
 * User: phillipesilva
 * Date: 16/12/2015
 * Time: 13:25
 */

namespace Service;

use Doctrine\ORM\EntityManager;
use React\Stomp\Factory;
use Util\InstanceProvider;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class AccessProfileControl {
    private $entityManager;
    private $profileFactory;

    /**
     * @param EntityManager $entityManager
     * @param \Factory\Profile $profileFactory
     */
    public function __construct(EntityManager $entityManager, \Factory\Profile $profileFactory)
    {
        $this->entityManager = $entityManager;
        $this->profileFactory = $profileFactory;
    }

    /**
     * @param $organizationId
     * @return mixed
     */
    public function findProfilesByOrganization($organizationId) {
        $profiles = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Profile')
            ->getProfileByOrganization($organizationId);
        return $profiles;
    }

    public function saveAccessProfileControl($control, $identityProfiles, $organizationId) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $savedProfileIds = [];
            foreach($identityProfiles as $identityProfile) {
                $profile = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Profile')
                    ->findOneBy(array('externalId' => $identityProfile, 'organization' => $organizationId));
                if (is_object($profile)) {
                    $profile->setControl($control);
                } else {
                    /** @var \Teknisa\Model\Geral\Entities\Perfilacesso $teknisaProfile */
                    $teknisaProfile = InstanceProvider::getRepository()->getPerfilacesso()
                        ->findOneBy(array('nrperfilacesso' =>$identityProfile));
                    $profile = $this->profileFactory->createProfile(null, $teknisaProfile->getNmperfilacesso(), 'A', null, $control);
                    $profile->setExternalId($identityProfile);
                }
                $savedProfileIds[] = $profile->getId();
                $this->entityManager->persist($profile);
            }
            $this->deleteNotNecessaryProfiles($control, $savedProfileIds, $organizationId);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function removeRules($profile)
    {
        $rules = InstanceProvider::getRepository()->getRule()
            ->findBy(array('profile' => $profile));
        foreach ($rules as $rule) {
            $this->entityManager->remove($rule);
        }
        $partnerProfiles = InstanceProvider::getRepository()->getPartnerProfile()
            ->findBy(array('profile' => $profile));
        foreach ($partnerProfiles as $partnerProfile) {
            $this->entityManager->remove($partnerProfile);
        }
        $this->entityManager->flush();
    }

    public function deleteNotNecessaryProfiles($control, $savedProfileIds, $organizationId) {
        /** @var Entities\Profile $profiles */
        $profiles = InstanceProvider::getRepository()->getProfile()
            ->findBy(array('control' => $control, 'organization' => $organizationId));
        foreach($profiles as $profile) {
            if(!in_array($profile->getId(), $savedProfileIds)) {
                $this->removeRules($profile);
                $this->entityManager->remove($profile);
            }
        }
    }

    /**
     * @return array
     */
    public function findPerfilAcesso()
    {
        $organizationId = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Profile $profileRepository */
        $profileRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Profile');
        return $profileRepository->findPerfilAcesso($organizationId);
    }
}